"""GenAI LLM model API handler."""

import json
import os

from litellm import completion
from pydantic import BaseModel

from mr_millionaire.libs.lib_constant import LLMConst


class LLMHandler:

    """LLM API handler."""

    def __init__(self) -> None:
        """Constructor for LLMHandler."""
        self.model = os.getenv(LLMConst.llm_model)

    def request_llm(self, request_prompt: str, structure: BaseModel = None) -> str | dict:
        """Request LLM model through API.

        Args:
            request_prompt (str): prompt to be sent to LLM.
            structure (BaseModel): expected output structure. Defaults to None.

        Returns:
            Union[str, dict]: response from LLM.

        """
        response = completion(
            model=self.model,
            messages=[self._construct_msg(prompt=request_prompt)],
            response_format=structure,
        )
        content = response.choices[0].message.content

        if structure:
            return json.loads(content)
        return content

    @staticmethod
    def _construct_msg(prompt: str) -> dict:
        """Construct llm message dict with prompt.

        Args:
            prompt (str): prompt to be sent to LLM.

        Returns:
            dict: message dict.

        """
        return {
            "role": "user",
            "content": prompt,
        }



"""Choice init module."""

from .config_manager import ConfigManager
from .game_runner import GameRunner
from .history_manager import HistoryManager
from .mem_manager import MemoryManager

__all__ = ["ConfigManager", "GameRunner", "HistoryManager", "MemoryManager"]



"""Quiz question schema."""


from pydantic import BaseModel, Field, ValidationInfo, conint, conlist, field_validator


class ConfigurationSchema(BaseModel):

    """Game configuraiton schema."""

    difficulty_level: conint(ge=1, le=5)  = Field(
        ..., description="Difficulty level of the question. ",
        )
    llm_api_key: str = Field(
        ..., description="LLM endpoint API key",
    )
    llm_model: str = Field(
        ..., description="LLM model to be used.",
    )


class FiftyFiftyAnswer(BaseModel):

    """Answer choices for the question."""

    choices: conlist(str, min_length=2, max_length=2) = Field(
        ..., description="Exactly two choices in which 1 is a correct answer.",
    )
    correct_answer: str = Field(
        ..., description="Correct answer (must match one of the choices).",
    )

    @field_validator("correct_answer")
    @classmethod
    def answer_must_be_in_choices(cls, v: str, info: ValidationInfo) -> str:
        """Field validator to verify the correct answer in choices."""
        if v not in info.data.get("choices", []):
            msg = "correct_answer must be one of the choices"
            raise ValueError(msg)
        return v


class PhoneFriendAnswer(BaseModel):

    """Answer to the phone friend question."""

    answer: str = Field(
        ..., description="Correct answer to the question.",
    )

class QuestionSchema(BaseModel):

    """A single multiple‑choice question with exactly four options."""

    question: str = Field(
        ..., description="Question shown to the player.",
    )
    choices: conlist(str, min_length=4, max_length=4) = Field(
        ..., description="Exactly four answer choices.",
    )
    correct_answer: str = Field(
        ..., description="Correct answer (must match one of the choices).",
    )


    @field_validator("correct_answer")
    @classmethod
    def answer_must_be_in_choices(cls, v: str, info: ValidationInfo) -> str:
        """Field validator to verify the correct answer in choices."""
        if v not in info.data.get("choices", []):
            msg = "correct_answer must be one of the choices"
            raise ValueError(msg)
        return v


class PlayerName(BaseModel):

    """Player name schema."""

    name: str = Field(
        ..., description="Name of the scientist.",
    )



"""History manager choice module."""

from datetime import datetime

from tabulate import tabulate

from mr_millionaire.libs import Choice
from mr_millionaire.libs.lib_constant import ConfigVal


class HistoryManager(Choice):

    """History manager choice class."""

    def run(self) -> None:
        """Run the History manager."""
        if ConfigVal.history_location.exists():
            with ConfigVal.history_location.open("r") as history:
                details = history.readlines()
                self._show_history(details)
        else:
            print("No Records found")

    @staticmethod
    def write_to_history(player_name: str, prize_won: int) -> None:
        """Write last player record to history.

        Args:
            player_name (str): Name of the player.
            prize_won (int): Prize money won.

        """
        date_time = datetime.now().strftime(ConfigVal.datetime_format)
        content = f"{date_time},{player_name},$ {prize_won}\n"
        mode = "w" if not ConfigVal.history_location.exists() else "a"
        with ConfigVal.history_location.open(mode) as history_f:
            history_f.write(content)


    @staticmethod
    def _show_history(details: list) -> None:
        """Shows the past history.

        Args:
            details (list): List of lines read from history file.

        """
        data = [entry.strip().split(",") for entry in details]
        print(tabulate(data, headers=ConfigVal.history_headers, tablefmt="grid"))



"""Game choice's Abstract."""

from abc import ABC, abstractmethod


class Choice(ABC):

    """Abstract class of choice."""

    @abstractmethod
    def run(self) -> None:
        """Run method to run the choice."""



"""Module to manage all the constant values used for this repository."""

from pathlib import Path
from typing import ClassVar


class Breaks:

    """Line breaks & Spaces."""

    newline: str = "\n"
    space_2: str = "  "
    strikethrough_start = "\033[9m"
    reset_formatting = "\033[0m"
    tab_2: str = "\t\t"


class ConfigVal:

    """Game configuration related values."""

    core_path = Path.home() / ".mr-millionaire"
    config_path = core_path / "config.json"

    datetime_format: str = "%m/%d/%Y, %H:%M:%S"
    default_configuration: ClassVar = {
        "difficulty": 2,
    }

    env_path = core_path / ".env"

    history_location = core_path / ".history"
    history_headers: ClassVar = ["Date-Time", "Player Name", "Won Amount"]

    memory = core_path / ".memory"


class Messages:

    """Messages used to be printed during this code execution."""

    default_int_err: str = "Expected integer value. Please retry. (Attempt left : {attempts_left})"
    constrain_err: str = "Only allowed values are {constrains}.\nPlease retry. (Attempt left : {attempts_left})"
    correct_answer: str = ("\nCongratulations {player}! Your answer is correct.\n"
                           "You've won - ${money}.")
    friend_response: str = "Your AI friend thinks the answer for the question : '{question}' could be '{answer}'."
    fifty_fifty_response: str = ("After shortlisting 50-50 process..\n"
                                 "Possible answers could be either '{choice_1}' or '{choice_2}'.\n")
    llm_config_msg: str = ("\nLLM functionality used in this game is by 'litellm' module.\n"
                        "environmental variables are needs to be set based on 'https://docs.litellm.ai/docs/providers'\n")
    max_attempt_reached: str = ("You have reached the maximum attempt limit."
                               "Max attempt reached. You must be 'Blind' or 'illiterate'.")
    no_lifeline: str = "\nNo lifeline available. You've already used all the lifelines available.\n"
    player_quit: str = ("That was a wise decision.!\n"
                        "{player} chose to quit the game and leaving with prize money - ${money}.")
    settings_header: str = "\nSettings Configuraiton\n"
    winning_message: str = "\nCongratulation {player}! You are a Millionaire now.!\nYou have won - ${money}"
    wrong_answer: str = ("\nOops! the answer wasn't correct. You lost the GAME! Mr. {player}\n"
                         "We are sorry to say you are not going to proceed further.! Better luck next time!.\n")


class LLMConst:

    """Open AI LLM constant values used in this repository."""

    api_key: str = "API_KEY"
    api_endpoint: str = "API_ENDPOINT"
    llm_model: str = "MODEL"


class GameValues:

    """Game values constant values used in this repository."""

    answer: str = "answer"
    choices: str = "choices"
    lifeline: tuple = ("l", "L")
    lifeline_options: ClassVar = {
        1: "Phone a Friend",
        2: "50-50",
    }
    player_name: str = "name"
    prize_money: ClassVar = {
        1: 100,  # One Hundred Dollars
        2: 200,  # Two Hundred Dollars
        3: 300,  # Three Hundred Dollars
        4: 500,  # Five Hundred Dollars
        5: 1_000,  # One Thousand Dollars
        6: 2_000,  # Two Thousand Dollars
        7: 4_000,  # Four Thousand Dollars
        8: 8_000,  # Eight Thousand Dollars
        9: 16_000,  # Sixteen Thousand Dollars
        10: 32_000,  # Thirty-Two Thousand Dollars
        11: 64_000,  # Sixty-Four Thousand Dollars
        12: 125_000,  # One Hundred Twenty-Five Thousand Dollars
        13: 250_000,  # Two Hundred Fifty Thousand Dollars
        14: 500_000,  # Five Hundred Thousand Dollars
        15: 1_000_000,  # One Million Dollars (the ultimate grand prize)
    }
    possible_answers: ClassVar = ["1", "2", "3", "4", "L", "l"]
    total_questions: int = 15


class LLMPrompts:

    """LLM prompts constant values used in this repository."""

    fifty_fifty_prompt: str = ("For the question '{question}'."
                               "Form the options : '{choices}', Select exactly two choices."
                               "One must be the correct answer, and one must be a wrong option."
                               "Return the result as a JSON with this format:"
                               "'choices' : ['<choice1>', '<choice2>'], correct_answer: '<correct answer>")
    in_memory_prompt: str = ("Some of the questions which the user already seen are as below.\n")
    phone_a_friend_prompt: str = ("Please give me an answer to the question : '{question}' which may have the "
                           "possible answers '{choices}'. What is the correct answer for this question.")
    player_name_prompt: str = "Give me a random scientist name."
    question_prompt: str = ("Generate a General Knowledge question with difficulty level as {difficulty_level} on "
                            "the scale of 1-5, also provide 4 multiple choice to pick the write answer.")
    question_history: str = "The question should not be in the below list of already asked questions."



"""Utility module for this repository."""
import sys
from typing import Any

from mr_millionaire.libs.lib_constant import Messages


def print_strikethrough(text: str, end: str = "\n") -> None:
    r"""Prints the given text with a strikethrough effect using ANSI escape codes.

    Args:
        text (str): The text to be printed.
        end (str, optional): The text to be printed. Defaults to "\n".

    """
    strikethrough_start = "\033[9m"
    reset_formatting = "\033[0m"

    print(f"{strikethrough_start}{text}{reset_formatting}", end=end)


def get_user_input(
        input_str: str,
        is_int: bool = False,
        constrains: list | tuple | None = None,
        max_attempt: int = 3,
        warning_msg: str | None = None,
        current_attempt: int = 1) -> int | str:
    """Get user input.

    Args:
        input_str (str): The text to be printed.
        is_int (bool, optional): Whether the input should be interpreted as an integer. Defaults to False.
        constrains (list | tuple, optional): A tuple of the possible constraints. Defaults to empty list.
        max_attempt (int, optional): The maximum number of attempts. Defaults to 3.
        warning_msg (str, optional): Warning message. Defaults to None.
        current_attempt (int, optional): Current attempt number. Defaults to 1.

    Returns:
        int | str: The user input.

    """
    if current_attempt > max_attempt:
        print(Messages.max_attempt_reached)
        sys.exit(1)
    if constrains is None:
        constrains = []
    user_input = input(input_str)
    if is_int:
        try:
            user_input = int(user_input)
        except ValueError:
            print(warning_msg if warning_msg
                  else Messages.default_int_err.format(attempts_left=max_attempt - current_attempt))
            current_attempt += 1
            return get_user_input(input_str, is_int, constrains, max_attempt, warning_msg, current_attempt)
    if constrains and user_input not in constrains:
        print(warning_msg if warning_msg
              else Messages.constrain_err.format(constrains=constrains, attempts_left=max_attempt - current_attempt))
        current_attempt += 1
        return get_user_input(input_str, is_int, constrains, max_attempt, warning_msg, current_attempt)
    return user_input


class DotDict(dict):

    """A dictionary subclass that supports dot notation access to dictionary attributes,.

    i.e., you can access keys as attributes.

    Example:
        data = DotDict({'name': 'user', 'config': {'enabled': True}})
        print(data.name)             # 'user'
        print(data.config.enabled)  # True

        Attributes can be set or deleted using dot notation as well.

    """

    def __getattr__(self, key: str) -> Any:
        """Get attribute using dot notation.

        If the value is a dictionary, it is converted to DotDict recursively.

        Args:
            key (str) : The key to retrieve from the dictionary.

        Returns:
            Any : The value associated with the key.

        Raises:
            AttributeError : If the key does not exist.

        """
        try:
            value = self[key]
            if isinstance(value, dict):
                return DotDict(value)
        except KeyError:
            msg = f"'DotDict' object has no attribute '{key}'"
            raise AttributeError(msg) from KeyError
        else:
            return value

    def __setattr__(self, key: str, value: str) -> None:
        """Set attribute using dot notation.

        Args:
            key (str): The key to set in the dictionary.
            value (str): The value to assign to the key.

        """
        self[key] = value

    def __delattr__(self, key: str) -> None:
        """Delete attribute using dot notation.

        Args:
            key (str) : The key to retrieve from the dictionary.

        Returns:
            Any : The value associated with the key.

        Raises:
            AttributeError : If the key does not exist.

        """
        try:
            del self[key]
        except KeyError:
            msg = f"'DotDict' object has no attribute '{key}'"
            raise AttributeError(msg) from KeyError




"""Memory DB handler."""

from mr_millionaire.libs.lib_constant import Breaks, ConfigVal


class MemoryHandler:

    """Memory DB handler."""

    def __init__(self) -> None:
        """Constructor for Memory handler."""
        self.found_in_memory = []

    def init_memory(self, force: bool = False) -> list:
        """Initialize the memory file.

        Args:
            force (bool): Flag to clear and initialize the memory again.

        Returns:
            list: List of questions in memory.

        """
        if ConfigVal.memory.exists() and not force:
            with ConfigVal.memory.open("r") as memory:
                return [q.strip() for q in memory.readlines()]
        with ConfigVal.memory.open("w") as memory:
            return []

    def add_to_memory(self, question: str) -> None:
        """Add the question to memory file.

        Args:
            question (str): Question to be added to memory.

        """
        with ConfigVal.memory.open("a") as memory:
            memory.write(question + Breaks.newline)

    def already_seen(self, question: str) -> bool:
        """Check if the question is already seen.

        Args:
            question (str): Question to be checked whether it is already there in memory.

        Returns:
            bool: True if the question found else False.

        """
        return question in self.init_memory()



"""Configuration handler module."""
import json

from dotenv import load_dotenv

from mr_millionaire.libs.lib_constant import Breaks, ConfigVal, Messages
from mr_millionaire.libs.utility import DotDict


class ConfigHandler:

    """Configuration handler class."""

    @property
    def configuration(self) -> dict:
        """Returns the current configuration as property.

        Returns:
            dict: Current configuration dictionary.

        """
        return DotDict(self.init_config())

    def init_config(self) -> dict:
        """Initialize configuration.

        Returns:
            dict: Initialized configuration dictionary.

        """
        if ConfigVal.config_path.exists():
            with ConfigVal.config_path.open("r") as f:
                return json.load(f)
        return {}

    def set_llm_config(self) -> None:
        """Set LLM related environmental var as configuration."""
        print(Messages.llm_config_msg)

        env_var_name = input("LLM Environment Variable Name : ")
        env_var_value = input("LLM Environment Variable Value : ")
        model_name = input("LLM Model (eg: groq/gemma2-9b-it) : ")

        env_content = f"{env_var_name.upper()}={env_var_value}\nMODEL={model_name}"

        with ConfigVal.env_path.open("w") as env_file:
            env_file.write(env_content)

        print(f"{Breaks.newline}Environment config files are saved to '{ConfigVal.env_path}' location.{Breaks.newline}")

    def set_configuration(self) -> None:
        """Set game configuration."""
        current_config = self.init_config()
        print(Messages.settings_header)
        configuration = current_config if current_config else ConfigVal.default_configuration
        new_configuration = {}

        for config_key, config_value in configuration.items():
            user_input = input(f"Configuration - {config_key} | Current - {config_value} | To Change : ")
            new_configuration[config_key] = user_input if user_input else config_value

        self._write_configuration(new_configuration)


    def set_initial_configuration(self) -> None:
        """Set required initial configuration on game run."""
        if not ConfigVal.core_path.exists():
            ConfigVal.core_path.mkdir(parents=True)

        if not ConfigVal.env_path.exists():
            self.set_llm_config()

        if not ConfigVal.config_path.exists():
            self.set_configuration()
        self._reload_env()

    @staticmethod
    def _reload_env() -> None:
        """Reload Environment variable."""
        load_dotenv(dotenv_path=ConfigVal.env_path)

    @staticmethod
    def _write_configuration(configuration: dict) -> None:
        """Write the game configuration to config file.

        Args:
            configuration (dict): Configuration dictionary.

        """
        with ConfigVal.config_path.open("w") as f:
            json.dump(configuration, f, indent=4)



"""Configuration manager choice module."""

from mr_millionaire.libs import Choice
from mr_millionaire.libs.config_handler import ConfigHandler
from mr_millionaire.libs.utility import get_user_input


class ConfigManager(Choice):

    """Configuration manager choice class."""

    def __init__(self) -> None:
        """Constructor for Configuration Manager."""
        self.handler = ConfigHandler()

    def run(self) -> None:
        """Run the configuration manager."""
        print("\nConfiguration Manager:\n")
        print("[1] LLM Configuration\t\t[2] Game Configuration")

        user_choice = get_user_input("\nChoice : ", is_int=True, constrains=range(1,2))
        if user_choice == 1:
            self.handler.set_llm_config()
        else:
            self.handler.set_configuration()



"""Main module to run the game."""


from dotenv import load_dotenv

from mr_millionaire.libs.choices import ConfigManager, GameRunner, HistoryManager, MemoryManager
from mr_millionaire.libs.lib_constant import Breaks, ConfigVal
from mr_millionaire.libs.utility import get_user_input

load_dotenv(dotenv_path=ConfigVal.env_path)


def choose_choice() -> int:
    """Header function to display available choices."""
    print(f"Choose one from options:{Breaks.newline}"
          f"{Breaks.space_2}[1] Play Game{Breaks.newline}"
          f"{Breaks.space_2}[2] Clear Memory{Breaks.newline}"
          f"{Breaks.space_2}[3] Configuration{Breaks.newline}"
          f"{Breaks.space_2}[4] Show History{Breaks.newline}")

    return get_user_input(
        "Enter your choice: ",
        is_int=True,
        constrains=list(range(1, 5)),
    )

def process_choice(choice: int) -> None:
    """Process the user choice.

    Args:
        choice (int): Index of the choice.

    """
    choice_processing = {
        1 : GameRunner(),
        2 : MemoryManager(),
        3 : ConfigManager(),
        4 : HistoryManager(),
    }
    process = choice_processing.get(choice)
    process.run()

def start_game() -> None:
    """Main function that starts the game."""
    print(f"Welcome to MillionAIRE!{Breaks.newline}")
    choice = choose_choice()
    process_choice(choice)


if __name__ == "__main__":
    start_game()



"""Memory manager choice module."""

from mr_millionaire.libs import Choice
from mr_millionaire.libs.memory_handler import MemoryHandler


class MemoryManager(Choice):

    """Memory manager choice class."""

    def __init__(self) -> None:
        """Constructor for the memory handler."""
        self.handler = MemoryHandler()

    def run(self) -> None:
        """Run the Memory manager."""
        print("\nClearing Memory...\n")
        self.handler.init_memory(force=True)
        print("\nMemory cleared successfully.\n")




"""Game runner choice module."""
import sys
from dataclasses import dataclass

from mr_millionaire.libs import Choice
from mr_millionaire.libs.choices.history_manager import HistoryManager
from mr_millionaire.libs.config_handler import ConfigHandler
from mr_millionaire.libs.lib_constant import Breaks, GameValues, LLMPrompts, Messages
from mr_millionaire.libs.llm_handler import LLMHandler
from mr_millionaire.libs.memory_handler import MemoryHandler
from mr_millionaire.libs.schema import FiftyFiftyAnswer, PhoneFriendAnswer, PlayerName, QuestionSchema
from mr_millionaire.libs.utility import get_user_input, print_strikethrough


@dataclass
class Question:

    """Question dataclass."""

    question_no: int
    question: str
    choices: list[str]
    correct_answer: str


class GameRunner(Choice):

    """Game runner choice class."""

    def __init__(self) -> None:
        """Constructor for GameRunner."""
        self.llm_handler = LLMHandler()
        self.config = ConfigHandler()
        self.history = HistoryManager()
        self.mem = MemoryHandler()
        self.prize_won = 0
        self._current_question = None
        self._question_history = []
        self._question_history_prompt = ""
        self._question_no = 0
        self._life_lines_used = {}
        self._player_name = ""


    def ask_question(self) -> None:
        """Get the question to the user.

        Returns:
            Question: next question to be asked to the player.

        """
        self._question_no += 1
        self._current_question = self.generate_next_question()
        self.mem.add_to_memory(self._current_question.question)
        self.show_question(self._current_question)
        self._question_history.append(self._current_question)

    def continue_or_drop(self) -> None:
        """Ask the user to continue or drop."""
        user_input = input(f"Press any key to continue or 'q' to leave with ${self.prize_won} : ")
        if user_input == "q":
            print(Messages.player_quit.format(player=self._player_name, money=self.prize_won))
            self.history.write_to_history(self._player_name, self.prize_won)
            sys.exit(1)

    def fifty_fifty(self) -> list:
        """Reduce the choice to 2 from 4.

        Returns:
            list: list of possible option.

        """
        fifty_fifty_prompt = LLMPrompts.fifty_fifty_prompt.format(
            question=self._current_question.question,
            choices=", ".join(self._current_question.choices),
        )
        fifty_fifty_answers = self.llm_handler.request_llm(
            fifty_fifty_prompt, FiftyFiftyAnswer,
        )
        return fifty_fifty_answers[GameValues.choices]

    def generate_next_question(self, *add_prompt: str) -> Question:
        """Generate next question to be asked to the player."""
        question_prompt = self._construct_question_prompt(*add_prompt)
        next_question = Question(
            question_no=self._question_no, **self.llm_handler.request_llm(question_prompt, QuestionSchema),
        )

        if self.mem.already_seen(next_question.question):
            self.mem.found_in_memory.append(next_question.question)
            return self.generate_next_question(*self.mem.found_in_memory)
        return next_question


    def phone_a_friend(self) -> str:
        """AI friend who can help you with an answer.

        Return:
            str: Answer for the question.

        """
        phone_a_friend_prompt = LLMPrompts.phone_a_friend_prompt.format(
            question=self._current_question.question,
            choices=", ".join(self._current_question.choices),
        )

        friend_answer = self.llm_handler.request_llm(
            phone_a_friend_prompt, PhoneFriendAnswer,
        )
        return friend_answer[GameValues.answer]

    def process_lifeline(self, ll_choice: int) -> None:
        """Process lifeline choice.

        Args:
            ll_choice (int): Life line choice chosen by user.

        """
        if GameValues.lifeline_options[ll_choice] == "Phone a Friend":
            friend_answer = self.phone_a_friend()
            print(Messages.friend_response.format(question=self._current_question.question, answer=friend_answer))
        if GameValues.lifeline_options[ll_choice] == "50-50":
            fifty_fifty_answer = self.fifty_fifty()
            print(Messages.fifty_fifty_response.format(choice_1=fifty_fifty_answer[0], choice_2=fifty_fifty_answer[1]))

    def run(self) -> None:
        """Run the game runner."""
        self.config.set_initial_configuration()
        self.set_player_name()
        while self._question_no < GameValues.total_questions:
            self.ask_question()
            self.continue_or_drop()
        print(Messages.winning_message.format(player=self._player_name, money=self.prize_won))
        self.history.write_to_history(self._player_name, self.prize_won)

    def show_lifelines(self) -> None:
        """Show lifelines available."""
        print(f"{Breaks.newline}Lifeline choices:")
        for ll_choice, lifeline in GameValues.lifeline_options.items():
            if ll_choice in self._life_lines_used:
                print_strikethrough(f"[{ll_choice}] {lifeline}", end=f"{Breaks.tab_2}")
            else:
                print(f"[{ll_choice}] {lifeline}", end=f"{Breaks.tab_2}")
        print(Breaks.newline)
        ll_value = get_user_input(
            "Enter your lifeline choice : ",
            is_int=True,
            constrains=[option for option in GameValues.lifeline_options if option not in self._life_lines_used],
        )
        self.process_lifeline(ll_value)
        self._life_lines_used[ll_value] = GameValues.lifeline_options.get(ll_value)

    def show_question(self, question: Question) -> None:
        """Show question in terminal.

        Args:
            question (Question): question to be shown.

        """
        print(f"{Breaks.newline}[Q.No {question.question_no}] {question.question}{Breaks.newline}"
              f"1. {question.choices[0]}{Breaks.tab_2}3. {question.choices[2]}{Breaks.newline}"
              f"2. {question.choices[1]}{Breaks.tab_2}4. {question.choices[3]}")

        answer = int(self.answer_lifeline_loop())
        self.process_answer(answer)

    def answer_lifeline_loop(self) -> str:
        """Allow user to answer the question with multiple lifelines.

        Returns:
            str: Answer for the question.

        """
        user_answer = self.request_answer_from_user()
        if user_answer in GameValues.lifeline:
            self.show_lifelines()
            return self.answer_lifeline_loop()
        return user_answer

    def process_answer(self, user_answer: int) -> None:
        """Validate user answer."""
        user_answer = self._current_question.choices[user_answer - 1]
        if user_answer.lower() == self._current_question.correct_answer.lower():
            self.prize_won = GameValues.prize_money.get(self._question_no)
            print(Messages.correct_answer.format(player=self._player_name, money=self.prize_won))
        else:
            print(Messages.wrong_answer.format(player=self._player_name))
            sys.exit(1)

    def request_answer_from_user(self) -> str | int:
        """Request answer from user.

        Returns:
            str | int: Answer for the question.

        """
        user_entry_constrains = GameValues.possible_answers
        if not self._lifeline_available:
            user_entry_constrains = list(filter(
                lambda x: x not in list(GameValues.lifeline), user_entry_constrains))

        return get_user_input(
            "Enter your answer or 'L' for lifeline: ",
            constrains=user_entry_constrains,
        )

    def set_player_name(self) -> None:
        """Set player name."""
        custom_player_name = input("Enter player name: ")
        if custom_player_name:
            self._player_name = custom_player_name
        else:
            self._player_name = self.llm_handler.request_llm(
                LLMPrompts.player_name_prompt, PlayerName,
            )[GameValues.player_name]

    def _construct_question_prompt(self, *add_prompt: str) -> str:
        """Construct question prompt.

        Returns:
            str: Question prompt.

        """
        base_prompt = LLMPrompts.question_prompt.format(
            difficulty_level=self.config.configuration.difficulty,
        )
        if self._question_history:
            last_question = self._question_history[-1]
            self._question_history_prompt += f"{last_question.question_no}. {last_question.question}\n"
            base_prompt += LLMPrompts.question_history + self._question_history_prompt
        if add_prompt:
            in_memory_questions = "\n".join(add_prompt)
            base_prompt += LLMPrompts.in_memory_prompt + in_memory_questions
        return base_prompt

    @property
    def _lifeline_available(self) -> bool:
        """Check if lifeline is available.

        Returns:
            bool: True if lifeline is available else False.

        """
        return len(self._life_lines_used.keys()) != len(GameValues.lifeline_options.keys())
